@@ -1,3 +1,6 @@
+from azure.durable_functions.models.actions.WhenAnyAction import WhenAnyAction
+from azure.durable_functions.models.actions.WhenAllAction import WhenAllAction
+from azure.durable_functions.models.ReplaySchema import ReplaySchema
 from datetime import datetime, timedelta
 from .orchestrator_test_utils \
     import assert_orchestration_state_equals, get_orchestration_state_result, assert_valid_schema
@@ -21,6 +24,26 @@ def generator_function(context):
 
     return outputs
 
+def generator_function_duplicate_yield(context):
+    task1 = context.call_activity("Hello", "Tokyo")
+    yield task1
+    yield task1
+
+    return ""
+
+def generator_function_compound_tasks(context):
+    yield context.call_activity("Hello", "Tokyo")
+
+    task1 = context.call_activity("Hello", "Tokyo")
+    task2 = context.call_activity("Hello", "Tokyo")
+    task3 = context.call_activity("Hello", "Tokyo")
+    task4 = context.task_any([task3])
+    task5 = context.task_all([task1, task2, task4])
+    task6 = context.task_any([task5])
+    yield task6
+
+    return ""
+
 def generator_function_time_is_not_none(context):
     outputs = []
 
@@ -105,15 +128,14 @@ def generator_function_new_guid(context):
     return outputs
 
 
-def base_expected_state(output=None) -> OrchestratorState:
-    return OrchestratorState(is_done=False, actions=[], output=output)
+def base_expected_state(output=None, replay_schema: ReplaySchema = ReplaySchema.V1) -> OrchestratorState:
+    return OrchestratorState(is_done=False, actions=[], output=output, replay_schema=replay_schema)
 
 
 def add_hello_action(state: OrchestratorState, input_: str):
     action = CallActivityAction(function_name='Hello', input_=input_)
     state.actions.append([action])
 
-
 def add_hello_completed_events(
         context_builder: ContextBuilder, id_: int, result: str):
     context_builder.add_task_scheduled_event(name='Hello', id_=id_)
@@ -385,3 +407,42 @@ def test_new_guid_orchestrator():
     # The two GUID lists should be the same
     assert outputs1 == outputs2
 
+def test_duplicate_yields_do_not_add_duplicate_actions():
+    """Tests that yield'ing a Task twice does not double the task's actions"""
+    context_builder = ContextBuilder('test_guid_orchestrator')
+    add_hello_completed_events(context_builder, 0, "\"Hello Tokyo!\"")
+
+    result = get_orchestration_state_result(
+        context_builder, generator_function_duplicate_yield)
+
+    expected_state = base_expected_state("")
+    add_hello_action(expected_state, 'Tokyo')
+    expected_state._is_done = True
+    expected = expected_state.to_json()
+
+    assert_valid_schema(result)
+    assert_orchestration_state_equals(expected, result)
+
+def test_compound_tasks_return_single_action_in_V2():
+    """Tests that compound tasks, in the v2 replay schema, are represented as a single "deep" action"""
+    context_builder = ContextBuilder('test_v2_replay_schema', replay_schema=ReplaySchema.V2)
+    add_hello_completed_events(context_builder, 0, "\"Hello Tokyo!\"")
+
+    result = get_orchestration_state_result(
+        context_builder, generator_function_compound_tasks)
+
+    expected_state = base_expected_state(replay_schema=ReplaySchema.V2)
+    expected_state._actions = [
+        [CallActivityAction("Hello", "Tokyo"), WhenAnyAction(
+            [WhenAllAction(
+                [CallActivityAction("Hello", "Tokyo"), CallActivityAction("Hello", "Tokyo"), WhenAnyAction(
+                    [CallActivityAction("Hello", "Tokyo")])
+                ])
+            ])
+        ]
+    ]
+    expected_state._is_done = False
+    expected = expected_state.to_json()
+
+    #assert_valid_schema(result)
+    assert_orchestration_state_equals(expected, result)
\ No newline at end of file
